@@ -1,6 +1,5 @@
 package Text::CSV;
 
-
 use strict;
 use Exporter;
 use Carp ();
@@ -8,14 +7,14 @@ use vars qw( $VERSION $DEBUG @ISA @EXPORT_OK %EXPORT_TAGS );
 @ISA = qw( Exporter );
 
 BEGIN {
-    $VERSION = '2.04';
+    $VERSION = '2.05';
     $DEBUG   = 0;
 }
 
 # if use CSV_XS, requires version
 my $Module_XS  = 'Text::CSV_XS';
 my $Module_PP  = 'Text::CSV_PP';
-my $XS_Version = '1.53';
+my $XS_Version = '1.59';
 
 my $Is_Dynamic = 0;
 
@@ -24,17 +23,17 @@ my @PublicMethods = qw/
     known_attributes
     PV IV NV CSV_TYPE_PV CSV_TYPE_IV CSV_TYPE_NV
     CSV_FLAGS_IS_QUOTED CSV_FLAGS_IS_BINARY CSV_FLAGS_ERROR_IN_FIELD CSV_FLAGS_IS_MISSING
-/;
+    /;
 
 %EXPORT_TAGS = (
     CONSTANTS => [qw(
-        CSV_FLAGS_IS_QUOTED
-        CSV_FLAGS_IS_BINARY
-        CSV_FLAGS_ERROR_IN_FIELD
-        CSV_FLAGS_IS_MISSING
-        CSV_TYPE_PV
-        CSV_TYPE_IV
-        CSV_TYPE_NV
+            CSV_FLAGS_IS_QUOTED
+            CSV_FLAGS_IS_BINARY
+            CSV_FLAGS_ERROR_IN_FIELD
+            CSV_FLAGS_IS_MISSING
+            CSV_TYPE_PV
+            CSV_TYPE_IV
+            CSV_TYPE_NV
     )],
 );
 @EXPORT_OK = (qw(csv PV IV NV), @{$EXPORT_TAGS{CONSTANTS}});
@@ -44,9 +43,9 @@ my @PublicMethods = qw/
 # Check the environment variable to decide worker module.
 
 unless ($Text::CSV::Worker) {
-    $Text::CSV::DEBUG and  Carp::carp("Check used worker module...");
+    $Text::CSV::DEBUG and Carp::carp("Check used worker module...");
 
-    if ( exists $ENV{PERL_TEXT_CSV} ) {
+    if (exists $ENV{PERL_TEXT_CSV}) {
         if ($ENV{PERL_TEXT_CSV} eq '0' or $ENV{PERL_TEXT_CSV} eq 'Text::CSV_PP') {
             _load_pp() or Carp::croak $@;
         }
@@ -70,7 +69,7 @@ sub new { # normal mode
     my $proto = shift;
     my $class = ref($proto) || $proto;
 
-    unless ( $proto ) { # for Text::CSV_XS/PP::new(0);
+    unless ($proto) { # for Text::CSV_XS/PP::new(0);
         return eval qq| $Text::CSV::Worker\::new( \$proto ) |;
     }
 
@@ -78,7 +77,7 @@ sub new { # normal mode
     #    Carp::croak("Can't set 'module' in non dynamic mode.");
     #}
 
-    if ( my $obj = $Text::CSV::Worker->new(@_) ) {
+    if (my $obj = $Text::CSV::Worker->new(@_)) {
         $obj->{_MODULE} = $Text::CSV::Worker;
         bless $obj, $class;
         return $obj;
@@ -87,7 +86,6 @@ sub new { # normal mode
         return;
     }
 
-
 }
 
 sub csv {
@@ -101,26 +99,22 @@ sub csv {
 
 sub require_xs_version { $XS_Version; }
 
-
 sub module {
     my $proto = shift;
-    return   !ref($proto)            ? $Text::CSV::Worker
-           :  ref($proto->{_MODULE}) ? ref($proto->{_MODULE}) : $proto->{_MODULE};
+    return !ref($proto)          ? $Text::CSV::Worker
+        : ref($proto->{_MODULE}) ? ref($proto->{_MODULE}) : $proto->{_MODULE};
 }
 
 *backend = *module;
 
-
 sub is_xs {
     return $_[0]->module eq $Module_XS;
 }
 
-
 sub is_pp {
     return $_[0]->module eq $Module_PP;
 }
 
-
 sub is_dynamic { $Is_Dynamic; }
 
 sub _load_xs { _load($Module_XS, $XS_Version) }
@@ -149,8 +143,6 @@ sub _load {
     return 1;
 }
 
-
-
 1;
 __END__
 
@@ -376,6 +368,13 @@ Return). The L<C<eol>|/eol> attribute cannot exceed 7 (ASCII) characters.
 If both C<$/> and L<C<eol>|/eol> equal C<"\015">, parsing lines that end on
 only a Carriage Return without Line Feed, will be L</parse>d correct.
 
+=head3 eol_type
+
+ my $eol = $csv->eol_type;
+
+This read-only method returns the internal state of  what is considered the
+valid EOL for parsing.
+
 =head3 sep_char
 
  my $csv = Text::CSV->new ({ sep_char => ";" });
@@ -479,6 +478,27 @@ so setting C<< { binary => 1 } >> is still a wise option.
 If this attribute is set to C<1>, any row that parses to a different number
 of fields than the previous row will cause the parser to throw error 2014.
 
+Empty rows or rows that result in no fields (like comment lines) are exempt
+from these checks.
+
+=head3 strict_eol
+
+ my $csv = Text::CSV->new ({ strict_eol => 1 });
+         $csv->strict_eol (0);
+ my $f = $csv->strict_eol;
+
+If this attribute is set to C<0>, no EOL consistency checks are done.
+
+If this attribute is set to C<1>, any row that parses with a EOL other than
+the EOL from the first row will cause a warning.  The error will be ignored
+and parsing continues. This warning is only thrown once.  Note that in data
+with various different line endings, C<\r\r> will still throw an error that
+cannot be ignored.
+
+If this attribute is set to C<2> or higher,  any row that parses with a EOL
+other than the EOL from the first row will cause error C<2016> to be thrown.
+The line being parsed to this error might not be stored in the result.
+
 =head3 skip_empty_rows
 
  my $csv = Text::CSV->new ({ skip_empty_rows => 1 });
@@ -1587,7 +1607,8 @@ Takes a list of scalar references to be used for output with  L</print>  or
 to store in the fields fetched by L</getline>.  When you do not pass enough
 references to store the fetched fields in, L</getline> will fail with error
 C<3006>.  If you pass more than there are fields to return,  the content of
-the remaining references is left untouched.
+the remaining references is left untouched.  Under C<strict> the two should
+match, otherwise L</getline> will fail with error C<2014>.
 
  $csv->bind_columns (\$code, \$name, \$price, \$description);
  while ($csv->getline ($fh)) {
@@ -1815,7 +1836,7 @@ error-input of L</getline>.
  $csv->error_diag ();
  $error_code               = 0  + $csv->error_diag ();
  $error_str                = "" . $csv->error_diag ();
- ($cde, $str, $pos, $rec, $fld) = $csv->error_diag ();
+ ($cde, $str, $pos, $rec, $fld, $xs) = $csv->error_diag ();
 
 If (and only if) an error occurred,  this function returns  the diagnostics
 of that error.
@@ -1831,7 +1852,9 @@ the byte at which the parsing failed in the current record. It might change
 to be the index of the current character in a later release. The records is
 the index of the record parsed by the csv instance. The field number is the
 index of the field the parser thinks it is currently  trying to  parse. See
-F<examples/csv-check> for how this can be used.
+F<examples/csv-check> for how this can be used. If C<$xs> is set, it is the
+line number in XS where the error was triggered (for debugging). C<XS> will
+show in void context only when L</diag_verbose> is set.
 
 If called in  scalar context,  it will return  the diagnostics  in a single
 scalar, a-la C<$!>.  It will contain the error code in numeric context, and
@@ -1840,6 +1863,18 @@ the diagnostics message in string context.
 When called as a class method or a  direct function call,  the  diagnostics
 are that of the last L</new> call.
 
+=head3 _cache_diag
+
+Note: This is an internal function only,  and output cannot be relied upon.
+Use at own risk.
+
+If debugging beyond what L</error_diag> is able to show, the internal cache
+can be shown with this function.
+
+ # Something failed ..
+ $csv->error_diag;
+ $csv->_cache_diag ();
+
 =head2 record_number
 
  $recno = $csv->record_number ();
@@ -1883,7 +1918,7 @@ This function is not exported by default and should be explicitly requested:
 
  use Text::CSV qw( csv );
 
-This is a high-level function that aims at simple (user) interfaces.  This
+This is a high-level function that aims at simple (user) interfaces.   This
 can be used to read/parse a C<CSV> file or stream (the default behavior) or
 to produce a file or write to a stream (define the  C<out>  attribute).  It
 returns an array- or hash-reference on parsing (or C<undef> on fail) or the
@@ -1893,6 +1928,11 @@ can get to the error using the class call to L</error_diag>
  my $aoa = csv (in => "test.csv") or
      die Text::CSV->error_diag;
 
+Note that failure here is the inability to start the parser,  like when the
+input does not exist or the arguments are unknown or conflicting.  Run-time
+parsing errors will return a valid reference, which can be empty, but still
+contains all results up till the error. See L</on_error>.
+
 This function takes the arguments as key-value pairs. This can be passed as
 a list or as an anonymous hash:
 
@@ -1907,6 +1947,7 @@ If not overridden, the default option used for CSV is
 
  auto_diag   => 1
  escape_null => 0
+ strict_eol  => 1
 
 The option that is always set and cannot be altered is
 
@@ -1966,18 +2007,23 @@ where, in the absence of the C<out> attribute, this is a shortcut to
 
 =head3 out
 
- csv (in => $aoa, out => "file.csv");
- csv (in => $aoa, out => $fh);
- csv (in => $aoa, out =>   STDOUT);
- csv (in => $aoa, out =>  *STDOUT);
- csv (in => $aoa, out => \*STDOUT);
- csv (in => $aoa, out => \my $data);
- csv (in => $aoa, out =>  undef);
- csv (in => $aoa, out => \"skip");
+ csv (in => $aoa,  out => "file.csv");
+ csv (in => $aoa,  out => $fh);
+ csv (in => $aoa,  out =>   STDOUT);
+ csv (in => $aoa,  out =>  *STDOUT);
+ csv (in => $aoa,  out => \*STDOUT);
+ csv (in => $aoa,  out => \my $data);
+ csv (in => $aoa,  out =>  undef);
+ csv (in => $aoa,  out => \"skip");
 
- csv (in => $fh,  out => \@aoa);
- csv (in => $fh,  out => \@aoh, bom => 1);
- csv (in => $fh,  out => \%hsh, key => "key");
+ csv (in => $fh,   out => \@aoa);
+ csv (in => $fh,   out => \@aoh, bom => 1);
+ csv (in => $fh,   out => \%hsh, key => "key");
+
+ csv (in => $file, out => $file);
+ csv (in => $file, out => $fh);
+ csv (in => $fh,   out => $file);
+ csv (in => $fh,   out => $fh);
 
 In output mode, the default CSV options when producing CSV are
 
@@ -2027,6 +2073,28 @@ collect that into a single data structure:
  my @list; # List of hashes
  csv (in => $_, out => \@list, bom => 1)    for sort glob "foo-[0-9]*.csv";
 
+=head4 Streaming
+
+If B<both> C<in> and C<out> are files, file handles or globs,  streaming is
+enforced by injecting an C<after_parse> callback  that immediately uses the
+L<C<say ()>|/say> method of the same instance to output the result and then
+rejects the record.
+
+If a C<after_parse> was already passed as attribute,  that will be included
+in the injected call. If C<on_in> was passed and C<after_parse> was not, it
+will be used instead. If both were passed, C<on_in> is ignored.
+
+The EOL of the first record of the C<in> source is consistently used as EOL
+for all records in the C<out> destination.
+
+The C<filter> attribute is not available.
+
+All other attributes are shared for C<in> and C<out>,  so you cannot define
+different encodings for C<in> and C<out>.  You need to pass a C<$fh>, where
+C<binmode> was used to apply the encoding layers.
+
+Note that this is work in progress and things might change.
+
 =head3 encoding
 
 If passed,  it should be an encoding accepted by the  C<:encoding()> option
@@ -2390,6 +2458,21 @@ will result in
   [ "1",   "2"       ],
   [ "3",   "4",  "5" ]]
 
+=head3 csv
+
+The I<function>  L</csv> can also be called as a method or with an existing
+Text::CSV object. This could help if the function is to be invoked a lot
+of times and the overhead of creating the object internally over  and  over
+again would be prevented by passing an existing instance.
+
+ my $csv = Text::CSV->new ({ binary => 1, auto_diag => 1 });
+
+ my $aoa = $csv->csv (in => $fh);
+ my $aoa = csv (in => $fh, csv => $csv);
+
+both act the same. Running this 20000 times on a 20 lines CSV file,  showed
+a 53% speedup.
+
 =head2 Callbacks
 
 Callbacks enable actions triggered from the I<inside> of Text::CSV.
@@ -2720,20 +2803,25 @@ C<$aoh> will be:
       }
     ]
 
-=item csv
+=item on_error
 
-The I<function>  L</csv> can also be called as a method or with an existing
-Text::CSV object. This could help if the function is to be invoked a lot
-of times and the overhead of creating the object internally over  and  over
-again would be prevented by passing an existing instance.
+This callback acts exactly as the L</error> hook.
 
- my $csv = Text::CSV->new ({ binary => 1, auto_diag => 1 });
+  my @err;
+  my $aoa = csv (in => $fh, on_error => sub { @err = @_ });
 
- my $aoa = $csv->csv (in => $fh);
- my $aoa = csv (in => $fh, csv => $csv);
+is identical to
 
-both act the same. Running this 20000 times on a 20 lines CSV file,  showed
-a 53% speedup.
+  my $aoa = csv (in => $fh, callbacks => {
+      error => sub { @err = @_ },
+      });
+
+It can be used for ignoring errors as well as for just keeping the error in
+case of analysis after the C<csv ()> function has returned.
+
+ my @err;
+ my $aoa = csv (in => "bad.csv, on_error => sub { @err = @_ });
+ die Text::CSV->error_diag if @err or !$aoa;
 
 =back
 
@@ -2937,6 +3025,11 @@ Inconsistent number of fields under strict parsing.
 
 An empty row was not allowed.
 
+=item *
+2016 "EOL - Inconsistent EOL"
+
+Inconsistent End-Of-Line detected under strict_eol parsing.
+
 =item *
 2021 "EIQ - NL char inside quotes, binary off"
 
